<?php
/**
 * DataTreeObject_File Class.
 *
 * $Horde: thor/lib/Item/File.php,v 1.3 2004/10/19 19:02:28 chuck Exp $
 *
 * @author  Carlos Pedrinaci <cpedrinaci@yahoo.es>
 * @version $Revision: 1.3 $
 * @package Thor
 */
class DataTreeObject_File extends DataTreeObject_Item {

    /**
     * Variable required for Dynamic Binding of Items.
     */
    var $_objectType = 'thor/File';

    var $_dir;

    function DataTreeObject_File($name)
    {
        parent::DataTreeObject_Item($name);
        $this->_dir = substr($name, 0,strrpos($name, ':'));
        $this->_dir = THOR_VFS . '/' . str_replace(':', '/', $this->_dir);
    }

    function _toAttributes()
    {
        if (!empty($this->data['file_upload_date'])) {
            $attributes[] = array('name'  => 'file',
                                  'key'   => 'upload_date',
                                  'value' => $this->data['file_upload_date']);
            $attributes[] = array('name'  => 'file',
                                  'key'   => 'uploader_id',
                                  'value' => $this->data['file_uploader_id']);
            $attributes[] = array('name'  => 'file',
                                  'key'   => 'name',
                                  'value' => $this->data['file_name']);
            $attributes[] = array('name'  => 'file',
                                  'key'   => 'type',
                                  'value' => $this->data['file_type']);
            $attributes[] = array('name'  => 'file',
                                  'key'   => 'size',
                                  'value' => $this->data['file_size']);
        }

        return array_merge(parent::_toAttributes(),$attributes);
    }

    function toArray()
    {
        $uploaded = !empty($this->data['file_name']);
        $array['file_name'] = ($uploaded) ? $this->data['file_name'] : null ;
        $array['file_type'] = ($uploaded) ? $this->data['file_type'] : null;
        $array['file_size'] = ($uploaded) ? $this->data['file_size'] : null;
        $array['file_uploader_id'] = ($uploaded) ? $this->data['file_uploader_id'] : null;
        $array['file_upload_date'] = ($uploaded) ? $this->data['file_upload_date'] : null;

        if ($uploaded) {
            $array['file_download_link'] = $this->createDownloadLink();
        }

        return array_merge(parent::toArray(), $array);
    }

    function fromArray(&$info)
    {
        global $notification;

        if (!empty($info['file'])) {
            $res = $this->saveFile($info['file']['name'], $info['file']['file']);
            if (is_a($res, 'PEAR_Error')) {
                $notification->push($res, 'horde.error');
            } else {
                $notification->push(sprintf(_("File received: %s"), $info['file']['name']), 'horde.success');
            }
            $this->data['file_name'] = $info['file']['name'];
            $this->data['file_type'] = $info['file']['type'];
            $this->data['file_size'] = $info['file']['size'];
            $this->data['file_uploader_id'] = Auth::getAuth();
            $this->data['file_upload_date'] = time();
        }

        parent::fromArray($info);
    }

    /**
     * Uploads the file
     */
    function saveFile($attachment_name, $attachment_file)
    {
        global $thor;

        $vfs = &$thor->getVFS();

        // FIrst of all delete the previous file if any
        $this->deleteFile();

        // Find a name
        while ($vfs->exists($this->_dir, $attachment_name)) {
            if (preg_match('/(.*)\[(\d+)\](\.[^.]*)?$/', $attachment_name, $match)) {
                $attachment_name = $match[1] . '[' . ++$match[2] . ']' . $match[3];
            } else {
                $dot = strrpos($attachment_name, '.');
                if ($dot === false) {
                    $attachment_name .= '[1]';
                } else {
                    $attachment_name = substr($attachment_name, 0, $dot) . '[1]' . substr($attachment_name, $dot);
                }
            }
        }

        // Write it
        $res = $vfs->write($this->_dir, $attachment_name, $attachment_file, true);
        if (is_a($res, 'PEAR_Error')) {
            return $res;
        }

        return true;
    }

    function deleteFile()
    {
        global $thor;

        $vfs = &$thor->getVFS();
        if (isset($this->data['file_name'])) {
            $res = $vfs->deleteFile($this->_dir, $this->data['file_name']);
             if (is_a($res, 'PEAR_Error')) {
                return $res;
            }
        }
        return true;
    }

    // Methods of Thor_Item to be implemented
    function showInformation()
    {
        global $prefs, $thor;

        $itemData = $this->toArray();
        $leftHeaderText = $this->getShortName();
        $project = $thor->_projectsContainer->getProject($itemData['project_id']);
        if ($project->isUserOfKind(Auth::getAuth(), USERS_ADMINS)) {
            $editUrl = Util::addParameter('item.php', array('item_id' => $itemData['item_id'],
                                                                                 'project_id' =>$itemData['project_id'],
                                                                                 'actionID' => EDIT_ITEM));

            $rightHeaderText = Horde::link(Horde::applicationUrl($editUrl), _("Edit Item")) . Horde::img('add.png', _("Edit Item"), 'hspace="2"');
        } else {
            $rightHeaderText = '';
        }
        $insertBreak = true;
        require THOR_TEMPLATES . '/section-header.inc';
        require THOR_TEMPLATES . '/view/File/description.inc';
        require THOR_TEMPLATES . '/view/footer.inc';
    }

    function download()
    {
        global $thor, $browser;

        $vfs = &$thor->getVFS();
        $file_data = $vfs->read($this->_dir, $this->data['file_name']);
        if (is_a($file_data, 'PEAR_Error')) {
            return $file_data;
        }
        $browser->downloadHeaders($this->data['file_name'], $this->data['file_type'], false, strlen($file_data));
        echo $file_data;
        return true;

    }

    function createDownloadLink()
    {
        global $mime_drivers, $mime_drivers_map;

        require_once 'Horde/MIME/Part.php';
        require_once 'Horde/MIME/Viewer.php';
        require_once 'Horde/MIME/Magic.php';
        require_once 'Horde/MIME/Contents.php';

        /* Set up the view url. */
        $url = Horde::applicationUrl('download.php');
        $params = array('file_type'  => $this->data['file_type'],
                                    'item_id'   => $this->getId(),
                                    'project_id' => $this->data['project_id'],
                                    'file_name'  => $this->data['file_name']);
        $url = Util::addParameter($url, $params);

        /* Make sure we have the tooltips javascript. */
        Horde::addScriptFile('tooltip.js', 'horde', true);

        /* Set up the icon and the title. */
        if (!empty($this->data['file_type'])) {
            $mime_icon = MIME_Viewer::getIcon($this->data['file_type']);
        } else {
            $mime_icon = MIME_Viewer::getIcon(MIME_Magic::filenameToMIME($this->data['file_name']));
        }
        $title = _("download") . ': ' . $this->data['file_name'];
        $tooltip = $title . "\n" . sprintf(_("size: %s"), $this->data['file_size']) . "\n" . sprintf(_("type: %s"), $this->data['file_type']);


        /* Return the link with a tooltip for further info on the download. */
        return Horde::linkTooltip($url, $title, '', '', '', $tooltip) . Horde::img($mime_icon, $title, 'align="middle"', '') . '&nbsp;' . $this->data['file_name'] . '</a>&nbsp;&nbsp;';
    }

    function initialize()
    {
        parent::initialize();
    }

    function _destroy()
    {
        $this->deleteFile();
        parent::_destroy();
    }

}

/**
 * FileForm Class
 *
 * $Horde: thor/lib/Item/File.php,v 1.3 2004/10/19 19:02:28 chuck Exp $
 *
 * @author  Carlos Pedrinaci <cpedrinaci@yahoo.es>
 * @version $Revision: 1.3 $
 * @package Thor
 */
class FileForm extends Horde_Form {

    var $_useFormToken = false;

    function FileForm(&$vars)
    {
        global $thor;

        $pid = $vars->get('project_id');
        $item_id = $vars->get('item_id');
        $title = empty($item_id) ? _("Create File - Step 2") : _("Edit File");
        parent::Horde_Form($vars, $title);

        /* Retrieve the project*/
        $project = &$thor->_projectsContainer->getProject($pid);
        $this->addVariable(_("File to upload"), 'file', 'file', false, false, null);
    }

}
