<?php

// Projects listing constants
/** @constant LIST_ALL_THOR List all the projects */
define('LIST_ALL_THOR', 0);
/** @constant LIST_USER_THOR Only show the user's projects */
define('LIST_USER_THOR', 1);

//Sorting
/** @const THOR_SORT_ASCEND Sort in ascending order.   */
define('THOR_SORT_ASCEND', 0);
/** @const THOR_SORT_DESCEND Sort in descending order. */
define('THOR_SORT_DESCEND', 1);

// Users sorting
/** @constant THOR_SORT_USERNAME Sort by user name */
define('THOR_SORT_USERNAME', 0);
/** @constant THOR_SORT_DATE Sort by join date */
define('THOR_SORT_DATE', 1);
/** @constant THOR_SORT_ROLE Sort by user role (privileges) */
define('THOR_SORT_ROLE', 2);

// Internal status flags
/** @constant USERS_ANY All the users (Admins, Devs, Accepted , Wanting, Refused and Discharged). */
define('USERS_ANY', 0);
/** @constant USERS_ADMINS All the admins of the project. */
define('USERS_ADMINS', 1);
/** @constant USERS_DEVS All the developers of the project.*/
define('USERS_DEVS', 2);
/** @constant USERS_TEAM All the developers and admins of the project.*/
define('USERS_TEAM', 4);
/** @constant USERS_WANTING All the users that have requested to contribute to the project. */
define('USERS_WANTING', 8);
/** @constant USERS_WANTING All the users that have been accepted but haven't yet performed the final step. */
define('USERS_ACCEPTED', 16);
/** @constant USERS_REFUSED All the users that have been refused.*/
define('USERS_REFUSED', 32);
/** @constant USERS_DISCHARGED Users that worked on the project */
define('USERS_DISCHARGED', 64);
/** @constant USERS_COORDINATOR Coordinator of the project */
define('USERS_COORDINATOR', 128);

// Constants for Team management actions: team.php
/** @constant UPGRADE_USER Set the user as a project admin. */
define('UPGRADE_USER', 0);
/** @constant DOWNGRADE_USER Set the user as a developer. */
define('DOWNGRADE_USER', 1);
/** @constant DISCHARGE_USER Discharge the user. */
define('DISCHARGE_USER', 2);
/** @constant ACCEPT_REQUEST Accepts the user request for contributing. */
define('ACCEPT_REQUEST', 4);
/** @constant REFUSE_REQUEST Refuses the user request for contributing. */
define('REFUSE_REQUEST', 8);
/** @constant REMOVE_USER Removes the user. Used when browsing refused users.*/
define('REMOVE_USER', 16);

// Constants for Project management actions: admin.php
/** @constant DELETE_PROJECT Delete the project */
define('DELETE_PROJECT', 0);
/** @constant BLOCK_PROJECT Block the project. */
define('BLOCK_PROJECT', 1);
/** @constant UNBLOCK_PROJECT Unblock the project. */
define('UNBLOCK_PROJECT', 2);
/** @constant ACCEPT_PROJECT Accepts the project. */
define('ACCEPT_PROJECT', 4);
/** @constant REFUSE_PROJECT Refuses the project. */
define('REFUSE_PROJECT', 8);

// Constants for Item management actions
/** @constant DELETE_ITEM Delete the item */
define('CREATE_ITEM', 0);
/** @constant DELETE_ITEM Delete the item */
define('EDIT_ITEM', 1);
/** @constant DELETE_ITEM Delete the item */
define('DELETE_ITEM', 2);

// Constants for Item and project write protection
/** @constant SET_WRITE_PROTECTED Write protect the item or the project. */
define('SET_WRITE_PROTECTED', 3);
/** @constant SET_WRITE_PROTECTED Unset write protection for the item or project. */
define('UNSET_WRITE_PROTECTED', 4);

// Flags Projects and Items status constants
/** @constant THOR_STATUS_ACTIVE The Project is under developed */
define('THOR_STATUS_ACTIVE', 1);
/** @constant THOR_STATUS_BLOCKED The Project is blocked */
define('THOR_STATUS_BLOCKED', 2);
/** @constant THOR_STATUS_REQUEST The Project or Item creation has been requested. Confirmation pending */
define('THOR_STATUS_REQUEST', 4);
/** @constant THOR_STATUS_EDITABLE For getting all BUT blocked projects */
define('THOR_STATUS_EDITABLE', 5);
/** @constant THOR_STATUS_ANY For getting any project (no matter its status) */
define('THOR_STATUS_ANY', 7);

// Flags for Item kinds kisting
/** @constant THOR_ITEMS_AVAILABLE For getting the available items */
define('THOR_ITEMS_AVAILABLE', 1);
/** @constant THOR_ITEMS_MANDATORY For getting the mandatory items */
define('THOR_ITEMS_MANDATORY', 2);

/** @constant THOR_VFS The virtual path to use for VFS data. */
define('THOR_VFS', '.horde/thor');

/**
 * Thor Base Class.
 *
 * $Horde: thor/lib/Thor.php,v 1.46 2004/12/07 16:09:17 chuck Exp $
 *
 * @author  Carlos Pedrinaci <cpedrinaci@yahoo.es>
 * @version $Revision: 1.46 $
 * @package Thor
 */
class Thor {

    /**
     * Hash storing information about the different projects.
     *
     * @var array $_projectKinds
     */
    var $_projectKinds = array();

    /**
     * Hash storing information about the different items.
     *
     * @var array $_itemKinds
     */
    var $_itemKinds = array();

    /**
     * The Projects Container
     */
    var $_projectsContainer;

    /**
     * The Items Container
     */
    var $_itemsContainer;

    /**
     * Constructor
     */
    function Thor()
    {
        global $conf, $registry;
        if (empty($conf['datatree']['driver'])) {
            Horde::fatal(_("You must configure a Horde DataTree backend to use Thor."), __FILE__, __LINE__);
        }
        /* Create the projects and items containers */
        $this->_projectsContainer = &Thor_Projects::singleton();
        $this->_itemsContainer = &Thor_ItemsContainer::singleton();
        /* Read Thor's registry */
        require_once THOR_BASE . '/config/thor_registry.php';
    }

    function &singleton()
    {
        static $thorOb;

        if (!isset($thorOb)) {
            $thorOb = new Thor();
        }

        return $thorOb;
    }

    /**
     * Sorts a list of DataTreeObjects.
     * @param array $array       The array to be sorted.
     * @param string $sortby         The key by which to sort. If not specified
     *                            then the first key is used.
     * @param integer $sortdir  Sort direction
     *                             0 = ascending (default)
     *                             1 = descending
     * @param boolean $associate  Keep key value association?
     */
    function sortObjects($array, $sortby = null, $sortdir = '0', $associate = true)
    {
        if (!is_array($array)) {
            return PEAR::raiseError('The function sortObjects requires an array.');
        }
        $sortedList = array();
        $keys = array_keys($array);
        foreach ($keys as $key) {
            if (!method_exists ($array[$key], 'toArray')) {
                return PEAR::raiseError('Method toArray() not implemented.');
            }
            $sortedList[$key] = $array[$key]->toArray();
        }

        require_once 'Horde/Array.php';
        Horde_Array::arraySort($sortedList, $sortby, $sortdir, $associate);
        return $sortedList;
    }

    /**
     * Formats a datatree, showing each child of a parent with appropriate
     * indent using '.. ' as a leader.
     *
     * @access public
     *
     * @param array $datatree         The datatree to format.
     * @param optional integer $baseLevel    The level number of the top, for formating in relative
     *
     * @return array  Formatted tree.
     */
    function formatDataTree($datatree, $baseLevel = 0)
    {
        if (count($datatree) == 0) {
            return array(DATATREE_ROOT => _("-- Top Level --"));
        }

        $baseLevel = $baseLevel + 1;
        foreach ($datatree as $object_id => $object) {
            $levels = explode(':', $object);
            $datatree[$object_id] = str_repeat('.. ', count($levels) - $baseLevel) . array_pop($levels);
        }

        $datatree = array(DATATREE_ROOT => _("-- Top Level --")) + $datatree;
        return $datatree;
    }

    /**
     * Returns the Project kinds as defined in thor_registry.php
     */
    function getProjectKinds()
    {
        return $this->_projectKinds;
    }

    /**
     * This just returns the kinds of projects and their names.
     * For using in forms
     */
    function getProjectNames()
    {
        $list = array();
        foreach ($this->_projectKinds as $pid => $pdata) {
            $list[$pid] = $pdata['name'];
        }
        return $list;
    }

    /**
     * Gets the different Item kinds available or mandatory  for a given project kind
     *
     * @param string $projectKind The Project kind
     * @param optional integer $type The kind of items to retrieve (available or mandatory)
     */
    function getItems($projectKind, $type = THOR_ITEMS_AVAILABLE)
    {
        $result = array();
        if (isset($projectKind) && isset($this->_projectKinds[$projectKind])) {
            if ($type & THOR_ITEMS_MANDATORY) {
                $itemsList = $this->_projectKinds[$projectKind]['mandatory_items'];
            } else {
                $itemsList = $this->_projectKinds[$projectKind]['available_items'];
            }

            foreach ($itemsList as $theItem) {
                $result[$theItem] = $this->_itemKinds[$theItem]['name'];
            }
        }

        return $result;
    }

    /**
     * Returns the Item kinds as defined in thor_registry.php
     */
    function getItemKinds()
    {
        return $this->_itemKinds;
    }

    /**
     * Gets the different Sub Item kinds available or mandatory  for a given item kind
     *
     * @param string $itemKind The Item kind
     * @param optional integer $type The kind of items to retrieve (available or mandatory)
     */
    function getSubItems($itemKind, $type = THOR_ITEMS_AVAILABLE)
    {
        $result = array();
        if (isset($itemKind) && isset($this->_itemKinds[$itemKind])) {
            if ($type & THOR_ITEMS_MANDATORY) {
                $itemsList = $this->_itemKinds[$itemKind]['mandatory_subItems'];
            } else {
                $itemsList = $this->_itemKinds[$itemKind]['available_subItems'];
            }

            foreach ($itemsList as $theItem) {
                $result[$theItem] = $this->_itemKinds[$theItem]['name'];
            }
        }

        return $result;
    }

    /**
    * Get the users of a project that match the criteria
    *
    * @param string $projectID The Project identifier.
    * @param constant integer $criteria The kind(s) of users to list
    * @param constant integer $sortby How to sort the results
    * @param constant integer $sortdir Which direction to sort the results
    * @result mixed Array of user names or False if there was a problem.
    */
    function listProjectUsers( $projectID, $criteria = USERS_ANY, $sortby, $sortdir)
    {
        if (empty($projectID)) {
            return false;
        }

        // Get the users by looking into the correct group(s)
        $project = &$this->_projectsContainer->getProject($projectID);
        if (!isset($project)) {
            return false;
        }

         $users = $project->listUsers($criteria);

         //Sort them
        return $users;
    }

    /**
     * This function gets the selected flags.
     *
     * @param array $flagsArray The array of flags
     * @param integer $value The value to be checked
     * @return array The array of flags selected
     */
    function getSelectedFlags($flagsArray, $value)
    {
        $result = array();
        foreach ($flagsArray as $flag => $text) {
            if ($value & $flag) {
                $result[] = $flag;
            }
        }
        return $result;
    }

    /**
     * Returns a {@link VFS} instance.
     *
     * @return VFS  A VFS instance.
     */
    function &getVFS()
    {
        global $conf;

        if (!isset($conf['vfs']['type'])) {
            return PEAR::raiseError(_("The VFS backend needs to be configured to enable uploads."), 'horde.error');
        }

        require_once 'VFS.php';
        return VFS::singleton($conf['vfs']['type'], Horde::getDriverConfig('vfs'));
    }

    /**
     * Build Thor's list of menu items.
     *
     * @access public
     */
    function getMenu($returnType = 'object')
    {
        global $registry, $thor;

        require_once 'Horde/Menu.php';
        $menu = &new Menu();

        $projectID = Util::getFormData('project_id');
        $itemID = Util::getFormData('item_id');

        /* Projects menu items. */
        $menu->add(Util::addParameter(Horde::applicationUrl('list.php'), 'actionID', LIST_ALL_THOR), _("All Projects"), 'thor.png',
                   null, '', null, (Util::getFormData('actionID') != LIST_USER_THOR) ? null : '__noselection');
        $menu->add(Util::addParameter(Horde::applicationUrl('list.php'), 'actionID', LIST_USER_THOR), _("My Projects"), 'thor.png',
                   null, '', null, (Util::getFormData('actionID') == LIST_USER_THOR) ? null : '__noselection');

        /* If we're doing something with a specific project, and the
         * current user can administer that project, show the editing
         * and team management links. */
        if ($projectID) {
            $project = &$thor->_projectsContainer->getProject($projectID);
            $url = Util::addParameter(Horde::applicationUrl('view.php'), 'project_id', $projectID);
            $menu->add($url, $project->getShortName(), 'thor.png');

            if ($project->isUserOfKind(Auth::getAuth(), USERS_ADMINS)) {
                $url = Util::addParameter('item.php', array('project_id' => $projectID,
                                                            'actionID' => CREATE_ITEM));

                if (isset($itemID)) {
                    $theItem = &$thor->_itemsContainer->getItem($itemID);
                    $itemData = $theItem->toArray();
                    $availKinds = $thor->getSubItems($itemData['item_kind']);
                    if (!empty($availKinds)) {
                        $url = Util::addParameter($url , 'item_parent_id', $itemID);
                        $menu->add(Horde::applicationUrl($url), _("Add Sub Item"), 'add.png');
                    }
                } else {
                    $menu->add(Horde::applicationUrl($url), _("Add Item"), 'add.png');
                }

                $url = Util::addParameter('team.php', 'project_id', $projectID);
                $menu->add(Horde::applicationUrl($url), _("Team Management"), 'group.png', $registry->getImageDir('horde'));
            }

            if ($project->isUserOfKind(Auth::getAuth(), USERS_TEAM)) {
                // Developer functions.
            }
        }

        $menu->add(Horde::applicationUrl('project.php'), _("New Project"), 'add.png',
                   null, '', null, isset($projectID) ? '__noselection' : null);

        if ($returnType == 'object') {
            return $menu;
        } else {
            return $menu->render();
        }
    }

}
