<?php
/**
 * File for listing projects.
 *
 * $Horde: thor/list.php,v 1.35 2005/01/03 14:35:33 jan Exp $
 *
 * Copyright 2003-2005 Carlos Pedrinaci <cpedrinaci@yahoo.es>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

@define('THOR_BASE', dirname(__FILE__));
require_once THOR_BASE . '/lib/base.php';
require_once 'Horde/UI/Tabs.php';
require_once 'Horde/UI/Pager.php';
require_once 'Horde/Variables.php';

/* Get the current action ID. */
$actionID = Util::getFormData('actionID', $prefs->getValue('listing'));

$vars = &Variables::getDefaultVariables();
if (!$vars->exists('status')) {
    $vars->set('status', THOR_STATUS_ACTIVE);
}

$status = $vars->get('status');
$projects_sortby = Util::getFormData('projects_sortby', $prefs->getValue('projects_sortby'));
$projects_sortdir = Util::getFormData('projects_sortdir', $prefs->getValue('projects_sortdir'));

switch ($actionID) {
case LIST_USER_THOR:
    /* Page variables. */
    $title = _("My Projects List");
    $listurl = Horde::applicationUrl(Util::addParameter('list.php', 'actionID', $actionID));
    $tabs = &new Horde_UI_Tabs('status', $vars);
    $tabs->addTab(_("Active"), $listurl, THOR_STATUS_ACTIVE);
    $tabs->addTab(_("Blocked"), $listurl, THOR_STATUS_BLOCKED);
    $tabs->addTab(_("Requests"), $listurl, THOR_STATUS_REQUEST);

    /* Get only the user projects list. */
    $role = ($status == THOR_STATUS_REQUEST) ? USERS_COORDINATOR : USERS_TEAM ;
    $projectsList = &$thor->_projectsContainer->listUserProjects(Auth::getAuth(), $status, $role, true);
    break;

case LIST_ALL_THOR:
default:
    /* Page variables. */
    $title = _("Projects List");

    switch ($conf['list']['view']) {
    case 'blocked':
        $listurl = Horde::applicationUrl(Util::addParameter('list.php', 'actionID', $actionID));
        $tabs = &new Horde_UI_Tabs('status', $vars);
        $tabs->addTab(_("Active"), $listurl, THOR_STATUS_ACTIVE);
        $tabs->addTab(_("Blocked"), $listurl, THOR_STATUS_BLOCKED);
        break;

    case 'all':
        $listurl = Horde::applicationUrl(Util::addParameter('list.php', 'actionID', $actionID));
        $tabs = &new Horde_UI_Tabs('status', $vars);
        $tabs->addTab(_("Active"), $listurl, THOR_STATUS_ACTIVE);
        $tabs->addTab(_("Blocked"), $listurl, THOR_STATUS_BLOCKED);
        $tabs->addTab(_("Requests"), $listurl, THOR_STATUS_REQUEST);
        break;
    }
    $projectsList = &$thor->_projectsContainer->getProjects($status, DATATREE_ROOT, -1, false, true);
    break;
}

if (is_a($projectsList, 'PEAR_Error')) {
    // Should not see that! -> use PERMS??
    $notification->push($projectsList, 'horde.error');
}

require THOR_TEMPLATES . '/common-header.inc';
require THOR_TEMPLATES . '/menu.inc';

if (isset($tabs)) {
    echo $tabs->render();
}

if (!is_a($projectsList, 'PEAR_Error')) {
    $projectsList = $thor->sortObjects($projectsList, $projects_sortby, $projects_sortdir);
    $num_projects = count($projectsList);

    $page = $vars->get('page');
    $viewurl = Util::addParameter('list.php', array(
            'actionID' => $actionID,
            'projects_sortby' => $projects_sortby,
            'projects_sortdir' => $projects_sortdir,
            'status' => $status
        ));

    $maxpage = $prefs->getValue('projects_maxpage');
    $perpage = $prefs->getValue('projects_perpage');

    $min = $page * $perpage;
    while ($min > $num_projects) {
        $page--;
        $min = $page * $perpage;
    }
    $projectsList = array_slice($projectsList, $min, $perpage);

    $pager = &new Horde_UI_Pager('page', $vars, array('num' => $num_projects,
                                                      'url' => $viewurl,
                                                      'page_count' => $maxpage,
                                                      'perpage' => $perpage));

    $leftHeaderText = $num_projects == 1 ? _("1 Project") : sprintf(_("%d Projects"), $num_projects);
    $rightHeaderText = '';
    $insertBreak = false;
    require THOR_TEMPLATES . '/section-header.inc';

    if ($num_projects) {
        require THOR_TEMPLATES . '/list/project_headers.inc';
        // List the projects.
        foreach ($projectsList as $project) {
            require THOR_TEMPLATES . '/list/project_summaries.inc';
        }
        require THOR_TEMPLATES . '/list/footer.inc';
    } else {
        require THOR_TEMPLATES . '/list/empty.inc';
    }
}

require THOR_TEMPLATES . '/list/footer.inc';

if (isset($pager)) {
    // Show the pager
    echo $pager->render();
}

require $registry->get('templates', 'horde') . '/common-footer.inc';
